@@ -9,7 +9,7 @@ function cleanup {
 trap "cleanup" EXIT
 
 # Check PEP-8 code style and McCabe complexity
-flake8 --show-pep8 --show-source .
+flake8 --statistics --show-source .
 
 # run tests
 nosetests --verbose --with-coverage
@@ -18,3 +18,7 @@ nosetests --verbose --with-coverage
 if [[ $TRAVIS_PYTHON_VERSION != 2.6 ]]; then
     pdoc --html --overwrite osmapi/OsmApi.py
 fi
+
+# setup a new virtualenv and try to install the lib
+virtualenv pyenv
+source pyenv/bin/activate && pip install .
