@@ -28,15 +28,11 @@ Find all information about changes of the different versions of this module
 """
 
 from __future__ import (absolute_import, print_function, unicode_literals)
-try:
-    import httplib
-except ImportError:
-    import http.client as httplib
-import base64
 import xml.dom.minidom
 import time
 import sys
 import urllib
+import requests
 from datetime import datetime
 
 from osmapi import __version__
@@ -140,7 +136,7 @@ class OsmApi:
             password=None,
             passwordfile=None,
             appid="",
-            created_by="osmapi/"+__version__,
+            created_by="osmapi/%s" % __version__,
             api="https://www.openstreetmap.org",
             changesetauto=False,
             changesetautotags={},
@@ -217,19 +213,19 @@ class OsmApi:
         self._changesetautodata = []
 
         # Get API
-        self._api = api
+        self._api = api.strip('/')
 
         # Get created_by
         if not appid:
             self._created_by = created_by
         else:
-            self._created_by = appid + " (" + created_by + ")"
+            self._created_by = "%s (%s)" % (appid, created_by)
 
         # Initialisation
         self._CurrentChangesetId = 0
 
         # Http connection
-        self._conn = self._get_http_connection()
+        self._session = self._get_http_session()
 
     def __del__(self):
         if self._changesetauto:
@@ -317,9 +313,9 @@ class OsmApi:
         If `NodeVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
         """
-        uri = "/api/0.6/node/"+str(NodeId)
+        uri = "/api/0.6/node/%s" % (NodeId)
         if NodeVersion != -1:
-            uri += "/"+str(NodeVersion)
+            uri += "/%s" % (NodeVersion)
         data = self._get(uri)
         if not data:
             return data
@@ -456,7 +452,7 @@ class OsmApi:
 
         `NodeId` is the unique identifier of a node.
         """
-        uri = "/api/0.6/node/"+str(NodeId)+"/history"
+        uri = "/api/0.6/node/%s/history" % NodeId
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -556,7 +552,8 @@ class OsmApi:
         `NodeIdList` is a list containing unique identifiers
         for multiple nodes.
         """
-        uri = "/api/0.6/nodes?nodes=" + ",".join([str(x) for x in NodeIdList])
+        node_list = ",".join([str(x) for x in NodeIdList])
+        uri = "/api/0.6/nodes?nodes=%s" % node_list
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -590,9 +587,9 @@ class OsmApi:
         If `WayVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
         """
-        uri = "/api/0.6/way/"+str(WayId)
+        uri = "/api/0.6/way/%s" % (WayId)
         if WayVersion != -1:
-            uri += "/"+str(WayVersion)
+            uri += "/%s" % (WayVersion)
         data = self._get(uri)
         if not data:
             return data
@@ -726,7 +723,7 @@ class OsmApi:
 
         `WayId` is the unique identifier of a way.
         """
-        uri = "/api/0.6/way/"+str(WayId)+"/history"
+        uri = "/api/0.6/way/%s/history" % (WayId)
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -793,7 +790,7 @@ class OsmApi:
 
         The `WayId` is a unique identifier for a way.
         """
-        uri = "/api/0.6/way/"+str(WayId)+"/full"
+        uri = "/api/0.6/way/%s/full" % (WayId)
         data = self._get(uri)
         return self.ParseOsm(data)
 
@@ -811,7 +808,8 @@ class OsmApi:
 
         `WayIdList` is a list containing unique identifiers for multiple ways.
         """
-        uri = "/api/0.6/ways?ways=" + ",".join([str(x) for x in WayIdList])
+        way_list = ",".join([str(x) for x in WayIdList])
+        uri = "/api/0.6/ways?ways=%s" % way_list
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -854,9 +852,9 @@ class OsmApi:
         If `RelationVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
         """
-        uri = "/api/0.6/relation/"+str(RelationId)
+        uri = "/api/0.6/relation/%s" % (RelationId)
         if RelationVersion != -1:
-            uri += "/"+str(RelationVersion)
+            uri += "/%s" % (RelationVersion)
         data = self._get(uri)
         if not data:
             return data
@@ -1017,7 +1015,7 @@ class OsmApi:
 
         `RelationId` is the unique identifier of a relation.
         """
-        uri = "/api/0.6/relation/"+str(RelationId)+"/history"
+        uri = "/api/0.6/relation/%s/history" % (RelationId)
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -1125,7 +1123,7 @@ class OsmApi:
 
         If you need all levels, use `OsmApi.RelationFullRecur`.
         """
-        uri = "/api/0.6/relation/"+str(RelationId)+"/full"
+        uri = "/api/0.6/relation/%s/full" % (RelationId)
         data = self._get(uri)
         return self.ParseOsm(data)
 
@@ -1145,7 +1143,7 @@ class OsmApi:
         for multiple relations.
         """
         relation_list = ",".join([str(x) for x in RelationIdList])
-        uri = "/api/0.6/relations?relations=" + relation_list
+        uri = "/api/0.6/relations?relations=%s" % relation_list
         data = self._get(uri)
         data = xml.dom.minidom.parseString(data)
         result = {}
@@ -1185,9 +1183,9 @@ class OsmApi:
         If `include_discussion` is set to `True` the changeset discussion
         will be available in the result.
         """
-        path = "/api/0.6/changeset/"+str(ChangesetId)
+        path = "/api/0.6/changeset/%s" % (ChangesetId)
         if (include_discussion):
-            path = path + "?include_discussion=true"
+            path += "?include_discussion=true"
         data = self._get(path)
         data = xml.dom.minidom.parseString(data)
         data = data.getElementsByTagName("osm")[0]
@@ -1209,7 +1207,7 @@ class OsmApi:
         if "created_by" not in ChangesetTags:
             ChangesetTags["created_by"] = self._created_by
         self._put(
-            "/api/0.6/changeset/" + str(self._CurrentChangesetId),
+            "/api/0.6/changeset/%s" % (self._CurrentChangesetId),
             self._XmlBuild("changeset", {"tag": ChangesetTags})
         )
         return self._CurrentChangesetId
@@ -1254,7 +1252,7 @@ class OsmApi:
         if not self._CurrentChangesetId:
             raise NoChangesetOpenError("No changeset currently opened")
         self._put(
-            "/api/0.6/changeset/"+str(self._CurrentChangesetId)+"/close",
+            "/api/0.6/changeset/%s/close" % (self._CurrentChangesetId),
             ""
         )
         CurrentChangesetId = self._CurrentChangesetId
@@ -1282,17 +1280,17 @@ class OsmApi:
         data += "<osmChange version=\"0.6\" generator=\""
         data += self._created_by + "\">\n"
         for change in ChangesData:
-            data += "<"+change["action"]+">\n"
+            data += "<" + change["action"] + ">\n"
             change["data"]["changeset"] = self._CurrentChangesetId
             data += self._XmlBuild(
                 change["type"],
                 change["data"],
                 False
             ).decode("utf-8")
-            data += "</"+change["action"]+">\n"
+            data += "</" + change["action"] + ">\n"
         data += "</osmChange>"
         data = self._post(
-            "/api/0.6/changeset/"+str(self._CurrentChangesetId)+"/upload",
+            "/api/0.6/changeset/%s/upload" % (self._CurrentChangesetId),
             data.encode("utf-8")
         )
         data = xml.dom.minidom.parseString(data)
@@ -1321,7 +1319,7 @@ class OsmApi:
                 'data': {}
             }
         """
-        uri = "/api/0.6/changeset/"+str(ChangesetId)+"/download"
+        uri = "/api/0.6/changeset/%s/download" % (ChangesetId)
         data = self._get(uri)
         return self.ParseOsc(data)
 
@@ -1371,7 +1369,7 @@ class OsmApi:
         if created_before:
             if not closed_after:
                 closed_after = "1970-01-01T00:00:00Z"
-            params["time"] = closed_after + "," + created_before
+            params["time"] = "%s,%s" % (closed_after, created_before)
         if only_open:
             params["open"] = 1
         if only_closed:
@@ -1420,7 +1418,7 @@ class OsmApi:
         """
         params = urllib.urlencode({'text': comment})
         data = self._post(
-            "/api/0.6/changeset/"+str(ChangesetId)+"/comment",
+            "/api/0.6/changeset/%s/comment" % (ChangesetId),
             params
         )
         data = xml.dom.minidom.parseString(data)
@@ -1457,7 +1455,7 @@ class OsmApi:
         """
         try:
             data = self._post(
-                "/api/0.6/changeset/"+str(ChangesetId)+"/subscribe",
+                "/api/0.6/changeset/%s/subscribe" % (ChangesetId),
                 None
             )
         except ApiError as e:
@@ -1499,7 +1497,7 @@ class OsmApi:
         """
         try:
             data = self._post(
-                "/api/0.6/changeset/"+str(ChangesetId)+"/unsubscribe",
+                "/api/0.6/changeset/%s/unsubscribe" % (ChangesetId),
                 None
             )
         except ApiError as e:
@@ -1828,10 +1826,10 @@ class OsmApi:
         if action == "create":
             if OsmData.get("id", -1) > 0:
                 raise OsmTypeAlreadyExistsError(
-                    "This "+OsmType+" already exists"
+                    "This %s already exists" % OsmType
                 )
             result = self._put(
-                "/api/0.6/" + OsmType + "/create",
+                "/api/0.6/%s/create" % OsmType,
                 self._XmlBuild(OsmType, OsmData)
             )
             OsmData["id"] = int(result.strip())
@@ -1839,14 +1837,14 @@ class OsmApi:
             return OsmData
         elif action == "modify":
             result = self._put(
-                "/api/0.6/" + OsmType + "/" + str(OsmData["id"]),
+                "/api/0.6/%s/%s" % (OsmType, OsmData["id"]),
                 self._XmlBuild(OsmType, OsmData)
             )
             OsmData["version"] = int(result.strip())
             return OsmData
         elif action == "delete":
             result = self._delete(
-                "/api/0.6/" + OsmType + "/" + str(OsmData["id"]),
+                "/api/0.6/%s/%s" % (OsmType, OsmData["id"]),
                 self._XmlBuild(OsmType, OsmData)
             )
             OsmData["version"] = int(result.strip())
@@ -1887,53 +1885,51 @@ class OsmApi:
             self._changesetautocpt = 0
         return None
 
-    def _http_request(self, cmd, path, auth, send):  # noqa
+    def _http_request(self, method, path, auth, send):  # noqa
+        """
+        Returns the response generated by an HTTP request.
+
+        `method` is a HTTP method to be executed
+        with the request data. For example: 'GET' or 'POST'.
+        `path` is the path to the requested resource relative to the
+        base API address stored in self._api. Should start with a
+        slash character to separate the URL.
+        `auth` is a boolean indicating whether authentication should
+        be preformed on this request.
+        `send` contains additional data that might be sent in a
+        request.
+        """
         if self._debug:
             error_msg = (
                 "%s %s %s"
-                % (time.strftime("%Y-%m-%d %H:%M:%S"), cmd, path)
+                % (time.strftime("%Y-%m-%d %H:%M:%S"), method, path)
             )
             print(error_msg, file=sys.stderr)
-        self._conn.putrequest(cmd, path)
-        self._conn.putheader('User-Agent', self._created_by)
+
+        # Add API base URL to path
+        path = self._api + path
+
+        user_pass = None
         if auth:
             try:
-                user_pass = self._username + ':' + self._password
+                user_pass = (self._username, self._password)
             except AttributeError:
                 raise UsernamePasswordMissingError("Username/Password missing")
 
-            try:
-                # Python 2
-                base64_user_pass = base64.encodestring(user_pass).strip()
-            except TypeError:
-                # Python 3
-                base64_user_pass = base64.encodestring(
-                    user_pass.encode('ascii')
-                    ).strip()
-                base64_user_pass = base64_user_pass.decode('utf-8')
-
-            self._conn.putheader(
-                'Authorization',
-                'Basic ' + base64_user_pass
-            )
-        if send is not None:
-            self._conn.putheader('Content-Length', len(send))
-        self._conn.endheaders()
-        if send:
-            self._conn.send(send)
-        response = self._conn.getresponse()
-        if response.status != 200:
-            payload = response.read().strip()
-            if response.status == 410:
+        response = self._session.request(method, path, auth=user_pass,
+                                         data=send)
+        if response.status_code != 200:
+            if response.status_code == 410:
                 return None
-            raise ApiError(response.status, response.reason, payload)
+            payload = response.content.strip()
+            raise ApiError(response.status_code, response.reason, payload)
         if self._debug:
             error_msg = (
                 "%s %s %s"
-                % (time.strftime("%Y-%m-%d %H:%M:%S"), cmd, path)
+                % (time.strftime("%Y-%m-%d %H:%M:%S"), method, path)
             )
             print(error_msg, file=sys.stderr)
-        return response.read()
+        return response.content
 
     def _http(self, cmd, path, auth, send):  # noqa
         i = 0
@@ -1947,7 +1943,7 @@ class OsmApi:
                         raise
                     if i != 1:
                         self._sleep()
-                    self._conn = self._get_http_connection()
+                    self._session = self._get_http_session()
                 else:
                     raise
             except Exception as e:
@@ -1960,17 +1956,17 @@ class OsmApi:
                     )
                 if i != 1:
                     self._sleep()
-                self._conn = self._get_http_connection()
+                self._session = self._get_http_session()
 
-    def _get_http_connection(self):
-        https_str = 'https://'
-        http_str = 'http://'
-        if self._api.lower().startswith(https_str):
-            return httplib.HTTPSConnection(self._api[len(https_str):], 443)
-        elif self._api.lower().startswith(http_str):
-            return httplib.HTTPConnection(self._api[len(http_str):], 80)
-        else:
-            return httplib.HTTPConnection(self._api, 80)
+    def _get_http_session(self):
+        """
+        Creates a requests session for connection pooling.
+        """
+        session = requests.Session()
+        session.headers.update({
+            'user-agent': self._created_by
+        })
+        return session
 
     def _sleep(self):
         time.sleep(5)
@@ -2189,7 +2185,7 @@ class OsmApi:
         # <tag... />
         for k, v in ElementData.get("tag", {}).items():
             xml += "    <tag k=\"" + self._XmlEncode(k)
-            xml += "\" v=\"" + self._XmlEncode(v)+"\"/>\n"
+            xml += "\" v=\"" + self._XmlEncode(v) + "\"/>\n"
 
         # <member... />
         for member in ElementData.get("member", []):
@@ -2200,7 +2196,7 @@ class OsmApi:
 
         # <nd... />
         for ref in ElementData.get("nd", []):
-            xml += "    <nd ref=\""+str(ref)+"\"/>\n"
+            xml += "    <nd ref=\"" + str(ref) + "\"/>\n"
 
         # </element>
         xml += "  </" + ElementType + ">\n"
