@@ -1,5 +1,4 @@
 from __future__ import (unicode_literals, absolute_import)
-from nose.tools import *  # noqa
 from . import osmapi_tests
 import osmapi
 import mock
@@ -43,21 +42,15 @@ def xmltosorteddict(xml):
     return recursive_sort(xml_dict)
 
 
-def debug(result):
-    from pprint import pprint
-    pprint(result)
-    assert_equals(0, 1)
-
-
 class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
     def test_ChangesetGet(self):
-        self._conn_mock()
+        self._session_mock()
 
         result = self.api.ChangesetGet(123)
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'GET')
-        self.assertEquals(args[1], '/api/0.6/changeset/123')
+        self.assertEquals(args[1], self.api_base + '/api/0.6/changeset/123')
 
         self.assertEquals(result, {
             'id': 123,
@@ -78,7 +71,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetUpdate(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -92,18 +85,17 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             }
         )
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'PUT')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1], self.api_base + '/api/0.6/changeset/4444')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osm version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osm version="0.6" generator="osmapi/0.8.1">\n'
                 b'  <changeset visible="true">\n'
                 b'    <tag k="test" v="foobar"/>\n'
-                b'    <tag k="created_by" v="osmapi/0.6.2"/>\n'
+                b'    <tag k="created_by" v="osmapi/0.8.1"/>\n'
                 b'  </changeset>\n'
                 b'</osm>\n'
             )
@@ -111,7 +103,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(result, 4444)
 
     def test_ChangesetUpdate_with_created_by(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -126,15 +118,14 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             }
         )
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'PUT')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1], self.api_base + '/api/0.6/changeset/4444')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osm version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osm version="0.6" generator="osmapi/0.8.1">\n'
                 b'  <changeset visible="true">\n'
                 b'    <tag k="test" v="foobar"/>\n'
                 b'    <tag k="created_by" v="MyTestOSMApp"/>\n'
@@ -145,7 +136,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(result, 4444)
 
     def test_ChangesetUpdate_wo_changeset(self):
-        self._conn_mock()
+        self._session_mock()
 
         with self.assertRaisesRegexp(
                 osmapi.NoChangesetOpenError,
@@ -157,7 +148,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             )
 
     def test_ChangesetCreate(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         result = self.api.ChangesetCreate(
             {
@@ -165,18 +156,17 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             }
         )
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'PUT')
-        self.assertEquals(args[1], '/api/0.6/changeset/create')
-        sendargs, kwargs = self.api._conn.send.call_args
+        self.assertEquals(args[1], self.api_base + '/api/0.6/changeset/create')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osm version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osm version="0.6" generator="osmapi/0.8.1">\n'
                 b'  <changeset visible="true">\n'
                 b'    <tag k="foobar" v="A new test changeset"/>\n'
-                b'    <tag k="created_by" v="osmapi/0.6.2"/>\n'
+                b'    <tag k="created_by" v="osmapi/0.8.1"/>\n'
                 b'  </changeset>\n'
                 b'</osm>\n'
             )
@@ -184,7 +174,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(result, 4321)
 
     def test_ChangesetCreate_with_created_by(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         result = self.api.ChangesetCreate(
             {
@@ -193,15 +183,14 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             }
         )
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'PUT')
-        self.assertEquals(args[1], '/api/0.6/changeset/create')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1], self.api_base + '/api/0.6/changeset/create')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osm version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osm version="0.6" generator="osmapi/0.8.1">\n'
                 b'  <changeset visible="true">\n'
                 b'    <tag k="foobar" v="A new test changeset"/>\n'
                 b'    <tag k="created_by" v="CoolTestApp"/>\n'
@@ -212,7 +201,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(result, 1234)
 
     def test_ChangesetCreate_with_open_changeset(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         self.api.ChangesetCreate(
             {
@@ -230,7 +219,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             )
 
     def test_ChangesetClose(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -240,12 +229,13 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
 
         self.api.ChangesetClose()
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'PUT')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444/close')
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/4444/close')
 
     def test_ChangesetClose_with_no_changeset(self):
-        self._conn_mock()
+        self._session_mock()
 
         with self.assertRaisesRegexp(
                 osmapi.NoChangesetOpenError,
@@ -253,7 +243,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             self.api.ChangesetClose()
 
     def test_ChangesetUpload_create_node(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -278,15 +268,15 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
 
         result = self.api.ChangesetUpload(changesdata)
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444/upload')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/4444/upload')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osmChange version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osmChange version="0.6" generator="osmapi/0.8.1">\n'
                 b'<create>\n'
                 b'  <node lat="47.123" lon="8.555" visible="true" '
                 b'changeset="4444">\n'
@@ -309,7 +299,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(result[0]['data']['version'], 1)
 
     def test_ChangesetUpload_modify_way(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -352,15 +342,15 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
 
         result = self.api.ChangesetUpload(changesdata)
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444/upload')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/4444/upload')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osmChange version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osmChange version="0.6" generator="osmapi/0.8.1">\n'
                 b'<modify>\n'
                 b'  <way id="4294967296" version="2" visible="true" '
                 b'changeset="4444">\n'
@@ -398,7 +388,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertEquals(data['version'], 3)
 
     def test_ChangesetUpload_delete_relation(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         # setup mock
         self.api.ChangesetCreate = mock.Mock(
@@ -436,15 +426,15 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
 
         result = self.api.ChangesetUpload(changesdata)
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/4444/upload')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/4444/upload')
         self.assertEquals(
-            xmltosorteddict(sendargs[0]),
+            xmltosorteddict(kwargs['data']),
             xmltosorteddict(
                 b'<?xml version="1.0" encoding="UTF-8"?>\n'
-                b'<osmChange version="0.6" generator="osmapi/0.6.2">\n'
+                b'<osmChange version="0.6" generator="osmapi/0.8.1">\n'
                 b'<delete>\n'
                 b'  <relation id="676" version="2" visible="true" '
                 b'changeset="4444">\n'
@@ -469,13 +459,14 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         self.assertNotIn('version', data)
 
     def test_ChangesetDownload(self):
-        self._conn_mock()
+        self._session_mock()
 
         result = self.api.ChangesetDownload(23123)
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, _ = self.api._session.request.call_args
         self.assertEquals(args[0], 'GET')
-        self.assertEquals(args[1], '/api/0.6/changeset/23123/download')
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/23123/download')
 
         self.assertEquals(len(result), 16)
         self.assertEquals(
@@ -500,15 +491,47 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
             }
         )
 
+    def test_ChangesetDownloadContainingUnicode(self):
+        self._session_mock()
+
+        # This changeset contains unicode tag values
+        # Note that the fixture data has been reduced from the
+        # original from openstreetmap.org
+        result = self.api.ChangesetDownload(37393499)
+
+        self.assertEquals(len(result), 2)
+        self.assertEquals(
+            result[1],
+            {
+                'action': 'create',
+                'type': 'way',
+                'data': {
+                    'changeset': 37393499,
+                    'id': 399491497,
+                    'nd': [4022271571, 4022271567, 4022271565],
+                    'tag': {'highway': 'service',
+                            # UTF-8 encoded 'LATIN SMALL LETTER O WITH STROKE'
+                            # Aka. 0xf8 in latin-1/ISO 8859-1
+                            'name': b'S\xc3\xb8nderskovvej'.decode('utf-8'),
+                            'service': 'driveway'},
+                    'timestamp': datetime.datetime(2016, 2, 23, 16, 55, 35),
+                    'uid': 328556,
+                    'user': 'InternationalUser',
+                    'version': 1,
+                    'visible': True
+                }
+            }
+        )
+
     def test_ChangesetsGet(self):
-        self._conn_mock()
+        self._session_mock()
 
         result = self.api.ChangesetsGet(
             only_closed=True,
             username='metaodi'
         )
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'GET')
         self.assertEquals(
             dict(urlparse.parse_qsl(urlparse.urlparse(args[1])[4])),
@@ -540,15 +563,15 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetGetWithComment(self):
-        self._conn_mock()
+        self._session_mock()
 
         result = self.api.ChangesetGet(52924, include_discussion=True)
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'GET')
         self.assertEquals(
             args[1],
-            '/api/0.6/changeset/52924?include_discussion=true'
+            self.api_base + '/api/0.6/changeset/52924?include_discussion=true'
         )
 
         self.assertEquals(result, {
@@ -590,19 +613,19 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetComment(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         result = self.api.ChangesetComment(
             123,
             comment="test comment"
         )
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/123/comment')
-        sendargs, _ = self.api._conn.send.call_args
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/123/comment')
         self.assertEquals(
-            sendargs[0],
+            kwargs['data'],
             "text=test+comment"
         )
         self.assertEquals(result, {
@@ -624,13 +647,14 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetSubscribe(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         result = self.api.ChangesetSubscribe(123)
 
-        args, _ = self.api._conn.putrequest.call_args
+        args, _ = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/123/subscribe')
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/123/subscribe')
         self.assertEquals(result, {
             'id': 123,
             'closed_at': datetime.datetime(2009, 9, 7, 22, 57, 37),
@@ -650,7 +674,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetSubscribeWhenAlreadySubscribed(self):
-        self._conn_mock(auth=True, status=409)
+        self._session_mock(auth=True, status=409)
 
         with self.assertRaises(osmapi.AlreadySubscribedApiError) as cm:
             self.api.ChangesetSubscribe(52924)
@@ -662,13 +686,14 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         )
 
     def test_ChangesetUnsubscribe(self):
-        self._conn_mock(auth=True)
+        self._session_mock(auth=True)
 
         result = self.api.ChangesetUnsubscribe(123)
 
-        args, kwargs = self.api._conn.putrequest.call_args
+        args, kwargs = self.api._session.request.call_args
         self.assertEquals(args[0], 'POST')
-        self.assertEquals(args[1], '/api/0.6/changeset/123/unsubscribe')
+        self.assertEquals(args[1],
+                          self.api_base + '/api/0.6/changeset/123/unsubscribe')
         self.assertEquals(result, {
             'id': 123,
             'closed_at': datetime.datetime(2009, 9, 7, 22, 57, 37),
@@ -688,7 +713,7 @@ class TestOsmApiChangeset(osmapi_tests.TestOsmApi):
         })
 
     def test_ChangesetUnsubscribeWhenNotSubscribed(self):
-        self._conn_mock(auth=True, status=404)
+        self._session_mock(auth=True, status=404)
 
         with self.assertRaises(osmapi.NotSubscribedApiError) as cm:
             self.api.ChangesetUnsubscribe(52924)
